@@ -22,7 +22,12 @@ import preludedb, CheetahFilters
 
 import prewikka.views
 from prewikka import view, Config, Log, Database, IDMEFDatabase, \
-     User, Auth, DataSet, Error, utils, siteconfig, localization
+     User, Auth, DataSet, Error, utils, siteconfig, localization, resolve
+
+try:
+    from threading import Lock
+except ImportError:
+    from dummy_threading import Lock
 
 
 class InvalidQueryError(Error.PrewikkaUserError):
@@ -92,29 +97,23 @@ def load_template(name, dataset):
 
 
 _core_cache = { }
-
-try:
-    import threading
-except ImportError:
-    _has_threads = False
-else:
-    _has_threads = True
-    _core_cache_lock = threading.Lock()
+_core_cache_lock = Lock()
 
 
 def get_core_from_config(path, threaded=False):
     global _core_cache
+    global _core_cache_lock
 
     if not path:
         path = siteconfig.conf_dir + "/prewikka.conf"
 
-    if _has_threads and threaded:
+    if threaded:
         _core_cache_lock.acquire()
 
     if not _core_cache.has_key(path):
         _core_cache[path] = Core(path)
 
-    if _has_threads and threaded:
+    if threaded:
         _core_cache_lock.release()
 
     return _core_cache[path]
@@ -127,9 +126,15 @@ class Core:
         self._env = Env()
         self._env.auth = None # In case of database error
         self._env.config = Config.Config(config)
+        self._env.log = Log.Log(self._env.config)
+        self._env.dns_max_delay = float(self._env.config.general.getOptionValue("dns_max_delay", 0))
         self._env.max_aggregated_source = int(self._env.config.general.getOptionValue("max_aggregated_source", 10))
         self._env.max_aggregated_target = int(self._env.config.general.getOptionValue("max_aggregated_target", 10))
         self._env.default_locale = self._env.config.general.getOptionValue("default_locale", None)
+
+        if self._env.dns_max_delay != -1:
+            resolve.init(self._env)
+
         preludedb.preludedb_init()
 
         self._database_schema_error = None
@@ -140,12 +145,12 @@ class Core:
             return
 
         self._env.idmef_db = IDMEFDatabase.IDMEFDatabase(self._env.config.idmef_database)
-        self._env.log = Log.Log(self._env.config)
         self._initHostCommands()
         self._loadViews()
         self._loadModules()
         self._initAuth()
 
+
     def _initDatabase(self):
         config = { }
         for key in self._env.config.database.keys():
@@ -350,6 +355,7 @@ class Core:
         # We check the character set after loading the template,
         # since the template might trigger a language change.
         dataset["document.charset"] = localization.getCurrentCharset()
+        resolve.process(self._env.dns_max_delay)
 
         try:
                 request.content = str(template)
