@@ -345,10 +345,17 @@ class ListedAlert(ListedMessage):
         if category == None:
             category = self._guessAddressCategory(address)
 
-        hfield = self.createHostField("alert.%s.node.address.address" % direction, address, category=category, direction=direction)
+        if dataset.has_key("no_dns"):
+            dns = False
+        else:
+            dns = True
+
+        hfield = self.createHostField("alert.%s.node.address.address" % direction, address,
+                                      category=category, direction=direction, dns=dns)
         dataset["addresses"].append(hfield)
 
     def _setMessageDirectionNodeName(self, dataset, direction, name):
+        dataset["no_dns"] = True
         dataset["addresses"].append(self.createHostField("alert.%s.node.name" % direction, name, direction=direction))
 
     def _setMessageDirectionOther(self, dataset, direction, path, value, extra_path=None, extra=None):
@@ -688,8 +695,8 @@ class AlertListing(MessageListing, view.View):
     def init(self, env):
         self._max_aggregated_classifications = int(env.config.general.getOptionValue("max_aggregated_classifications", 10))
 
-    def _getMessageIdents(self, criteria, limit=-1, offset=-1):
-        return self.env.idmef_db.getAlertIdents(criteria, limit, offset)
+    def _getMessageIdents(self, criteria, limit=-1, offset=-1, order_by="time_desc"):
+        return self.env.idmef_db.getAlertIdents(criteria, limit, offset, order_by)
 
     def _fetchMessage(self, ident):
         return self.env.idmef_db.getAlert(ident)
@@ -884,7 +891,7 @@ class AlertListing(MessageListing, view.View):
                 if classification:
                     entry_param["classification_object_%d" % self.parameters.max_index] = "alert.classification.text"
                     entry_param["classification_operator_%d" % self.parameters.max_index] = "="
-                    entry_param["classification_value_%d" % self.parameters.max_index] = utils.escape_criteria(classification)
+                    entry_param["classification_value_%d" % self.parameters.max_index] = classification
 
                 entry_param["alert.assessment.impact.severity"] = severity or "none"
                 entry_param["alert.assessment.impact.completion"] = completion or "none"
@@ -910,8 +917,16 @@ class AlertListing(MessageListing, view.View):
         ag_list = ag_s + ag_t + ag_c + ag_a
 
         ##
-        selection = [ "%s/group_by" % path for path in ag_list ] + \
-                    [ "count(alert.create_time)", "max(alert.create_time)/order_desc" ]
+        selection = [ "%s/group_by" % path for path in ag_list ]
+
+        if self.parameters["orderby"] == "time_asc":
+            selection += [ "count(alert.create_time)", "max(alert.create_time)/order_asc" ]
+        elif self.parameters["orderby"] == "time_desc":
+            selection += [ "count(alert.create_time)", "max(alert.create_time)/order_desc" ]
+        elif self.parameters["orderby"] == "count_desc":
+            selection += [ "count(alert.create_time)/order_desc", "max(alert.create_time)" ]
+        elif self.parameters["orderby"] == "count_asc":
+            selection += [ "count(alert.create_time)/order_asc", "max(alert.create_time)" ]
 
         use_sensor_localtime = self.parameters["timezone"] == "sensor_localtime"
         if not use_sensor_localtime:
@@ -1034,7 +1049,6 @@ class AlertListing(MessageListing, view.View):
         return parameters
 
     def _setMessages(self, criteria):
-        self.dataset["messages"] = [ ]
         self.dataset["aggregated_source"] = self.parameters["aggregated_source"]
         self.dataset["aggregated_target"] = self.parameters["aggregated_target"]
         self.dataset["aggregated_classification"] = self.parameters["aggregated_classification"]
@@ -1052,14 +1066,7 @@ class AlertListing(MessageListing, view.View):
         if len(ag_s + ag_t + ag_c + ag_a) > 0:
             return self._setAggregatedMessagesNoValues(criteria, ag_s, ag_t, ag_c, ag_a)
 
-        results = self.env.idmef_db.getAlertIdents(criteria)
-
-        for ident in results[self.parameters["offset"] : self.parameters["offset"] + self.parameters["limit"]]:
-            message = self.env.idmef_db.getAlert(ident)
-            dataset = self._setMessage(message, ident)
-            self.dataset["messages"].append(dataset)
-
-        return len(results)
+        return MessageListing._setMessages(self, criteria)
 
     def _setDatasetConstants(self):
         self.dataset["classification_filters"] = CLASSIFICATION_FILTERS
